Package: TreeTools
Title: Create, Modify and Analyse Phylogenetic Trees
Version: 2.4.0
Authors@R: c(
  person("Martin R.", 'Smith', role = c("aut", "cre", "cph"), 
         email = "martin.smith@durham.ac.uk",
         comment = c(ORCID = "0000-0001-5660-1727")),
  person(given = "Emmanuel", family = "Paradis", role = c("cph"),
         comment = c(ORCID = "0000-0003-3092-2199", "ape library")),
  person(given = "Robert", family = "Noble", role = c("cph"),
         comment = c(ORCID = "0000-0002-8057-4252", "RUtreebalance"))
  )
License: GPL (>= 3)
Copyright: Incorporates C/C++ code from 'ape' by Emmanuel Paradis
        <doi:10.1093/bioinformatics/bty633> and 'RUtreebalance' by
        Robert Noble <doi:10.5281/zenodo.5873857>.
Description: Efficient implementations of functions for the creation, 
  modification and analysis of phylogenetic trees.
  Applications include:
  generation of trees with specified shapes;
  tree rearrangement;
  analysis of tree shape;
  rooting of trees and extraction of subtrees;
  calculation and depiction of split support;
  plotting the position of rogue taxa (Klopfstein & Spasojevic 2019)
  <doi:10.1371/journal.pone.0212942>;
  calculation of ancestor-descendant relationships,
  of 'stemwardness' (Asher & Smith, 2022) <doi:10.1093/sysbio/syab072>,
  and of tree balance (Mir et al. 2013, Lemant et al. 2022)
  <doi:10.1016/j.mbs.2012.10.005>, <doi:10.1093/sysbio/syac027>;
  artificial extinction (Asher & Smith, 2022) <doi:10.1093/sysbio/syab072>;
  import and export of trees from Newick, Nexus (Maddison et al. 1997)
  <doi:10.1093/sysbio/46.4.590>,
  and TNT <https://www.lillo.org.ar/phylogeny/tnt/> formats;
  and analysis of splits and cladistic information.
URL: https://ms609.github.io/TreeTools/,
        https://github.com/ms609/TreeTools/
BugReports: https://github.com/ms609/TreeTools/issues/
SystemRequirements: C++17
Depends: R (>= 3.6.0), ape (>= 5.6),
Imports: bit64, fastmatch (>= 1.1.3), methods, PlotTools, Rdpack (>=
        2.6.6),
Suggests: RCurl, spelling, knitr, phangorn (>= 2.2.1), Rcpp (>= 1.0.8),
        rmarkdown, testthat (>= 3.0), TreeDist, TreeSearch, vdiffr (>=
        1.0.0),
Config/Needs/benchmark: bench, TreeDist
Config/Needs/check: rcmdcheck, testthat
Config/Needs/coverage: covr
Config/Needs/memcheck: pkgdown, testthat
Config/Needs/metadata: codemeta
Config/Needs/revdeps: revdepcheck
Config/Needs/website: pkgdown
Config/roxygen2/version: 8.0.0
Config/testthat/parallel: false
Config/testthat/edition: 3
LinkingTo: Rcpp
RdMacros: Rdpack
LazyData: true
ByteCompile: true
Encoding: UTF-8
Language: en-GB
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2026-06-02 15:02:39 UTC; pjjg18
Author: Martin R. Smith [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0001-5660-1727>),
  Emmanuel Paradis [cph] (ORCID: <https://orcid.org/0000-0003-3092-2199>,
    ape library),
  Robert Noble [cph] (ORCID: <https://orcid.org/0000-0002-8057-4252>,
    RUtreebalance)
Maintainer: Martin R. Smith <martin.smith@durham.ac.uk>
Repository: CRAN
Date/Publication: 2026-06-02 16:50:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-06-02 16:55:25 UTC; unix
Archs: TreeTools.so.dSYM
