# Generated by roxygen2: do not edit by hand

S3method("!",Splits)
S3method("&",Splits)
S3method("+",Splits)
S3method("-",Splits)
S3method("<",MixedBase)
S3method("<",phylo)
S3method("==",MixedBase)
S3method("==",phylo)
S3method(">",MixedBase)
S3method(">",phylo)
S3method("[",phyDat)
S3method("[[",Splits)
S3method("names<-",Splits)
S3method("|",Splits)
S3method(.Edge,list)
S3method(.Edge,numeric)
S3method(.Edge,phylo)
S3method(.UpdateNodeLabel,numeric)
S3method(.UpdateNodeLabel,phylo)
S3method(AllTipLabels,"NULL")
S3method(AllTipLabels,Splits)
S3method(AllTipLabels,TreeNumber)
S3method(AllTipLabels,list)
S3method(AllTipLabels,matrix)
S3method(AllTipLabels,multiPhylo)
S3method(AllTipLabels,phylo)
S3method(ApePostorder,"NULL")
S3method(ApePostorder,list)
S3method(ApePostorder,multiPhylo)
S3method(ApePostorder,phylo)
S3method(ArtificialExtinction,matrix)
S3method(ArtificialExtinction,phyDat)
S3method(Cherries,numeric)
S3method(Cherries,phylo)
S3method(Cladewise,"NULL")
S3method(Cladewise,list)
S3method(Cladewise,matrix)
S3method(Cladewise,multiPhylo)
S3method(Cladewise,phylo)
S3method(CladisticInfo,Splits)
S3method(CladisticInfo,list)
S3method(CladisticInfo,multiPhylo)
S3method(CladisticInfo,phylo)
S3method(CollapseNode,phylo)
S3method(ConsensusWithout,"NULL")
S3method(ConsensusWithout,list)
S3method(ConsensusWithout,multiPhylo)
S3method(ConsensusWithout,phylo)
S3method(DropTip,"NULL")
S3method(DropTip,Splits)
S3method(DropTip,list)
S3method(DropTip,multiPhylo)
S3method(DropTip,phylo)
S3method(EdgeRatio,phylo)
S3method(KeepTip,"NULL")
S3method(KeepTip,Splits)
S3method(KeepTip,list)
S3method(KeepTip,multiPhylo)
S3method(KeepTip,phylo)
S3method(KeptPaths,data.frame)
S3method(KeptPaths,matrix)
S3method(KeptVerts,numeric)
S3method(KeptVerts,phylo)
S3method(LongBranch,"NULL")
S3method(LongBranch,list)
S3method(LongBranch,multiPhylo)
S3method(LongBranch,phylo)
S3method(MakeTreeBinary,"NULL")
S3method(MakeTreeBinary,list)
S3method(MakeTreeBinary,multiPhylo)
S3method(MakeTreeBinary,phylo)
S3method(MinimumSpanningTree,"NULL")
S3method(MinimumSpanningTree,dist)
S3method(MinimumSpanningTree,matrix)
S3method(NSplits,"NULL")
S3method(NSplits,ClusterTable)
S3method(NSplits,Splits)
S3method(NSplits,character)
S3method(NSplits,list)
S3method(NSplits,multiPhylo)
S3method(NSplits,numeric)
S3method(NSplits,phylo)
S3method(NTip,Splits)
S3method(NTip,default)
S3method(NTip,list)
S3method(NTip,matrix)
S3method(NTip,multiPhylo)
S3method(NTip,phyDat)
S3method(NTip,phylo)
S3method(NodeDepth,"NULL")
S3method(NodeDepth,list)
S3method(NodeDepth,matrix)
S3method(NodeDepth,multiPhylo)
S3method(NodeDepth,phylo)
S3method(NodeOrder,"NULL")
S3method(NodeOrder,list)
S3method(NodeOrder,matrix)
S3method(NodeOrder,multiPhylo)
S3method(NodeOrder,phylo)
S3method(Postorder,"NULL")
S3method(Postorder,list)
S3method(Postorder,multiPhylo)
S3method(Postorder,numeric)
S3method(Postorder,phylo)
S3method(PostorderOrder,"NULL")
S3method(PostorderOrder,numeric)
S3method(PostorderOrder,phylo)
S3method(Preorder,"NULL")
S3method(Preorder,list)
S3method(Preorder,multiPhylo)
S3method(Preorder,numeric)
S3method(Preorder,phylo)
S3method(Pruningwise,"NULL")
S3method(Pruningwise,list)
S3method(Pruningwise,multiPhylo)
S3method(Pruningwise,phylo)
S3method(RenumberTips,"NULL")
S3method(RenumberTips,Splits)
S3method(RenumberTips,list)
S3method(RenumberTips,multiPhylo)
S3method(RenumberTips,phylo)
S3method(Reweight,matrix)
S3method(Reweight,phyDat)
S3method(RootNode,"NULL")
S3method(RootNode,list)
S3method(RootNode,multiPhylo)
S3method(RootNode,numeric)
S3method(RootNode,phylo)
S3method(RootNodeDistance,"NULL")
S3method(RootNodeDistance,character)
S3method(RootNodeDistance,numeric)
S3method(RootOnNode,"NULL")
S3method(RootOnNode,list)
S3method(RootOnNode,multiPhylo)
S3method(RootOnNode,phylo)
S3method(RootTree,"NULL")
S3method(RootTree,list)
S3method(RootTree,matrix)
S3method(RootTree,multiPhylo)
S3method(RootTree,phylo)
S3method(RootedTreeWithShape,integer64)
S3method(RootedTreeWithShape,numeric)
S3method(SisterSize,"NULL")
S3method(SisterSize,character)
S3method(SisterSize,numeric)
S3method(SortTree,"NULL")
S3method(SortTree,list)
S3method(SortTree,multiPhylo)
S3method(SortTree,phylo)
S3method(SplitImbalance,"NULL")
S3method(SplitImbalance,Splits)
S3method(SplitImbalance,phylo)
S3method(SplitInformation,Splits)
S3method(SplitInformation,numeric)
S3method(SplitInformation,phylo)
S3method(SplitsInBinaryTree,"NULL")
S3method(SplitsInBinaryTree,Splits)
S3method(SplitsInBinaryTree,default)
S3method(SplitsInBinaryTree,list)
S3method(SplitsInBinaryTree,multiPhylo)
S3method(SplitsInBinaryTree,numeric)
S3method(SplitsInBinaryTree,phylo)
S3method(TCIContext,numeric)
S3method(TCIContext,phylo)
S3method(TipLabels,MixedBase)
S3method(TipLabels,Splits)
S3method(TipLabels,TreeNumber)
S3method(TipLabels,character)
S3method(TipLabels,default)
S3method(TipLabels,list)
S3method(TipLabels,logical)
S3method(TipLabels,matrix)
S3method(TipLabels,multiPhylo)
S3method(TipLabels,numeric)
S3method(TipLabels,phyDat)
S3method(TipLabels,phylo)
S3method(TipsInSplits,Splits)
S3method(TipsInSplits,phylo)
S3method(TntOrder,"NULL")
S3method(TntOrder,list)
S3method(TntOrder,multiPhylo)
S3method(TntOrder,numeric)
S3method(TntOrder,phylo)
S3method(TopologyOnly,default)
S3method(TopologyOnly,list)
S3method(TopologyOnly,multiPhylo)
S3method(TopologyOnly,phylo)
S3method(TotalCopheneticIndex,"NULL")
S3method(TotalCopheneticIndex,list)
S3method(TotalCopheneticIndex,multiPhylo)
S3method(TotalCopheneticIndex,phylo)
S3method(Treeness,"NULL")
S3method(Treeness,list)
S3method(Treeness,multiPhylo)
S3method(Treeness,phylo)
S3method(UnrootTree,"NULL")
S3method(UnrootTree,list)
S3method(UnrootTree,multiPhylo)
S3method(UnrootTree,phylo)
S3method(WriteTntCharacters,matrix)
S3method(WriteTntCharacters,phyDat)
S3method(as.ClusterTable,"NULL")
S3method(as.ClusterTable,list)
S3method(as.ClusterTable,multiPhylo)
S3method(as.ClusterTable,phylo)
S3method(as.MixedBase,"NULL")
S3method(as.MixedBase,MixedBase)
S3method(as.MixedBase,TreeNumber)
S3method(as.MixedBase,integer64)
S3method(as.MixedBase,multiPhylo)
S3method(as.MixedBase,numeric)
S3method(as.MixedBase,phylo)
S3method(as.Newick,list)
S3method(as.Newick,multiPhylo)
S3method(as.Newick,phylo)
S3method(as.Splits,Splits)
S3method(as.Splits,character)
S3method(as.Splits,integer)
S3method(as.Splits,list)
S3method(as.Splits,logical)
S3method(as.Splits,matrix)
S3method(as.Splits,multiPhylo)
S3method(as.Splits,numeric)
S3method(as.Splits,phylo)
S3method(as.TreeNumber,"NULL")
S3method(as.TreeNumber,MixedBase)
S3method(as.TreeNumber,TreeNumber)
S3method(as.TreeNumber,character)
S3method(as.TreeNumber,multiPhylo)
S3method(as.TreeNumber,phylo)
S3method(as.character,Splits)
S3method(as.integer64,TreeNumber)
S3method(as.logical,Splits)
S3method(as.logical,phylo)
S3method(as.matrix,ClusterTable)
S3method(as.matrix,phyDat)
S3method(as.matrix,phylo)
S3method(as.multiPhylo,Splits)
S3method(as.multiPhylo,list)
S3method(as.multiPhylo,phyDat)
S3method(as.multiPhylo,phylo)
S3method(as.phylo,MixedBase)
S3method(as.phylo,Splits)
S3method(as.phylo,TreeNumber)
S3method(as.phylo,integer64)
S3method(as.phylo,numeric)
S3method(c,Splits)
S3method(duplicated,Splits)
S3method(head,Splits)
S3method(length,Splits)
S3method(names,Splits)
S3method(print,ClusterTable)
S3method(print,Splits)
S3method(print,TreeNumber)
S3method(rev,Splits)
S3method(sort,Splits)
S3method(sort,multiPhylo)
S3method(summary,ClusterTable)
S3method(summary,Splits)
S3method(t,Splits)
S3method(tail,Splits)
S3method(unique,Splits)
S3method(xtfrm,Splits)
export(.CompatibleRaws)
export(.CompatibleSplit)
export(.RandomParent)
export(.UnrootedKeys)
export(AddTip)
export(AddTipEverywhere)
export(AddUnconstrained)
export(AllAncestors)
export(AllTipLabels)
export(AncestorEdge)
export(ApePostorder)
export(ApeTime)
export(ArtEx)
export(ArtificialExtinction)
export(BalancedTree)
export(CharacterInformation)
export(Cherries)
export(CladeSizes)
export(Cladewise)
export(CladisticInfo)
export(CladisticInformation)
export(CollapseEdge)
export(CollapseNode)
export(CompatibleSplits)
export(Consensus)
export(ConsensusWithout)
export(ConstrainedNJ)
export(Decompose)
export(DescendantEdges)
export(DescendantTips)
export(DoubleFactorial)
export(DoubleFactorial64)
export(DropTip)
export(DropTipPhylo)
export(EdgeAncestry)
export(EdgeDistances)
export(EdgeRatio)
export(EndSentence)
export(ExtractTaxa)
export(FirstMatchingSplit)
export(Hamming)
export(IC1Spr)
export(ImposeConstraint)
export(J1Index)
export(JQIndex)
export(KeepTip)
export(KeepTipPostorder)
export(KeepTipPreorder)
export(KeptPaths)
export(KeptVerts)
export(LabelSplits)
export(LeafLabelInterchange)
export(ListAncestors)
export(LnDoubleFactorial)
export(LnDoubleFactorial.int)
export(LnRooted)
export(LnRooted.int)
export(LnSplitMatchProbability)
export(LnTreesMatchingSplit)
export(LnTreesMatchingTree)
export(LnUnrooted)
export(LnUnrooted.int)
export(LnUnrootedMult)
export(LnUnrootedSplits)
export(LnUnrootedTreesMatchingSplit)
export(Log2DoubleFactorial)
export(Log2Rooted)
export(Log2Rooted.int)
export(Log2TreesMatchingSplit)
export(Log2TreesMatchingTree)
export(Log2Unrooted)
export(Log2Unrooted.int)
export(Log2UnrootedMult)
export(Log2UnrootedSplits)
export(Log2UnrootedTreesMatchingSplit)
export(LogDoubleFactorial)
export(LogDoubleFactorial.int)
export(LongBranch)
export(MRCA)
export(MSTEdges)
export(MSTLength)
export(MakeTreeBinary)
export(MarkMissing)
export(MatchEdges)
export(MatchNodes)
export(MatchStrings)
export(MatrixToPhyDat)
export(MorphoBankDecode)
export(MrBayesTrees)
export(MultiSplitInformation)
export(N1Spr)
export(NDescendants)
export(NJTree)
export(NPartitionPairs)
export(NPartitions)
export(NRooted)
export(NRooted64)
export(NRootedShapes)
export(NSplits)
export(NTip)
export(NUnrooted)
export(NUnrooted64)
export(NUnrootedMult)
export(NUnrootedShapes)
export(NUnrootedSplits)
export(NewickTree)
export(NeworderPhylo)
export(NeworderPruningwise)
export(NexusTokens)
export(NexusTokensToInteger)
export(NodeDepth)
export(NodeNumbers)
export(NodeOrder)
export(PaintTree)
export(PairwiseDistances)
export(PathLengths)
export(PectinateTree)
export(PhyDat)
export(PhyDatToMatrix)
export(PhyDatToString)
export(PhyToString)
export(PhydatToString)
export(PolarizeSplits)
export(Postorder)
export(PostorderOrder)
export(Preorder)
export(Pruningwise)
export(RandomTree)
export(ReadAsPhyDat)
export(ReadCharacters)
export(ReadMrBayes)
export(ReadMrBayesTrees)
export(ReadNotes)
export(ReadTNTAsPhyDat)
export(ReadTNTCharacters)
export(ReadTntAsPhyDat)
export(ReadTntCharacters)
export(ReadTntTree)
export(Renumber)
export(RenumberEdges)
export(RenumberTips)
export(RenumberTree)
export(Reweight)
export(RightmostCharacter)
export(RoguePlot)
export(RootNode)
export(RootNodeDist)
export(RootNodeDistance)
export(RootOnNode)
export(RootTree)
export(RootedTreeShape)
export(RootedTreeWithShape)
export(SampleOne)
export(SingleTaxonTree)
export(SisterSize)
export(SortTree)
export(SplitConflicts)
export(SplitConsistent)
export(SplitFrequency)
export(SplitImbalance)
export(SplitInformation)
export(SplitMatchProbability)
export(SplitsInBinaryTree)
export(StarTree)
export(Stemminess)
export(StringToPhyDat)
export(Subsplit)
export(Subtree)
export(SupportColor)
export(SupportColour)
export(TCIContext)
export(TNTOrder)
export(TNTText2Tree)
export(TipLabels)
export(TipTimedTree)
export(TipsInSplits)
export(TntOrder)
export(TntText2Tree)
export(TopologyOnly)
export(TotalCopheneticIndex)
export(TreeIsRooted)
export(Treeness)
export(TreesMatchingSplit)
export(TreesMatchingTree)
export(TrivialSplits)
export(Unquote)
export(UnrootTree)
export(UnrootedKeys)
export(UnrootedTreeKey)
export(UnrootedTreeShape)
export(UnrootedTreeWithKey)
export(UnrootedTreeWithShape)
export(UnrootedTreesMatchingSplit)
export(UnshiftTree)
export(WithoutTrivialSplits)
export(WriteTntCharacters)
export(YuleTree)
export(ZeroTaxonTree)
export(as.ClusterTable)
export(as.MixedBase)
export(as.Newick)
export(as.Splits)
export(as.TreeNumber)
export(as.multiPhylo)
export(doubleFactorials)
export(edge_to_splits)
export(is.TreeNumber)
export(logDoubleFactorials)
export(replicate64)
export(root_on_node)
export(sapply64)
export(vapply64)
exportMethods("%in%")
exportMethods(match)
exportMethods(xor)
importFrom(PlotTools,SpectrumLegend)
importFrom(Rdpack,reprompt)
importFrom(ape,all.equal.phylo)
importFrom(ape,as.phylo)
importFrom(ape,cophenetic.phylo)
importFrom(ape,edgelabels)
importFrom(ape,is.rooted)
importFrom(ape,multi2di)
importFrom(ape,nj)
importFrom(ape,read.nexus)
importFrom(ape,read.tree)
importFrom(ape,write.tree)
importFrom(bit64,as.integer64)
importFrom(bit64,integer64)
importFrom(fastmatch,"%fin%")
importFrom(fastmatch,fmatch)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grDevices,hcl)
importFrom(grDevices,rgb)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setMethod)
importFrom(methods,setOldClass)
importFrom(stats,as.dist)
importFrom(stats,cophenetic)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,runif)
importFrom(stats,setNames)
importFrom(utils,combn)
importFrom(utils,globalVariables)
importFrom(utils,head)
importFrom(utils,tail)
useDynLib(TreeTools, .registration = TRUE)
