#' TreeTools
#'
#' "TreeTools" is an R package that provides functions for creating, modifying and
#' analysing phylogenetic trees.
#' It complements packages such as
#' [\pkg{ape}](https://cran.r-project.org/package=ape),
#' [\pkg{phangorn}](https://cran.r-project.org/package=phangorn) and
#' [\pkg{phytools}](https://cran.r-project.org/package=phytools),
#' aiming for efficient and robust implementations of functions, typically
#' applied to unweighted trees (i.e. those without edge lengths).
#'
#' Full documentation is available [online](https://ms609.github.io/TreeTools/).
#'
#' @keywords internal
"_PACKAGE"

# Suppress "NOTE: Nothing imported from Rdpack":
#' @importFrom Rdpack reprompt
NULL

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
## usethis namespace: end
NULL
