% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/Cherries.R
\name{Cherries}
\alias{Cherries}
\alias{Cherries.phylo}
\alias{Cherries.numeric}
\title{Count cherries in a tree}
\usage{
Cherries(tree, nTip)

\method{Cherries}{phylo}(tree, nTip = NTip(tree))

\method{Cherries}{numeric}(tree, nTip)
}
\arguments{
\item{tree}{A binary tree, of class \code{phylo}; or a matrix corresponding to its
edge matrix.}

\item{nTip}{Number of leaves in tree.}
}
\value{
\code{Cherries()} returns an integer specifying the number of nodes whose
children are both leaves.

Equations for the number of cherries expected under uniform and Yule tree
models are derived by \insertCite{McKenzie2000;textual}{TreeTools}.
}
\description{
\code{Cherries()} counts the number of vertices in a binary tree whose children
are both leaves.
}
\references{
\insertAllCited{}
}
\seealso{
Other tree properties:
\code{\link[=ConsensusWithout]{ConsensusWithout()}},
\code{\link[=EdgeRatio]{EdgeRatio()}},
\code{\link[=LongBranch]{LongBranch()}},
\code{\link[=MatchEdges]{MatchEdges()}},
\code{\link[=NSplits]{NSplits()}},
\code{\link[=NTip]{NTip()}},
\code{\link[=NodeNumbers]{NodeNumbers()}},
\code{\link[=PathLengths]{PathLengths()}},
\code{\link[=SplitsInBinaryTree]{SplitsInBinaryTree()}},
\code{\link[=TipLabels]{TipLabels()}},
\code{\link[=TreeIsRooted]{TreeIsRooted()}},
\code{\link[=Treeness]{Treeness()}}
}
\author{
\href{https://orcid.org/0000-0001-5660-1727}{Martin R. Smith}
(\href{mailto:martin.smith@durham.ac.uk}{martin.smith@durham.ac.uk})
}
\concept{tree properties}
